/*
 * Copyright (c) 2018 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution  and  use  in  source  and binary  forms,  with  or  without
 * modification, are permitted provided that the following conditions are met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 * THE SOFTWARE  IS PROVIDED "AS IS"  AND THE AUTHOR  DISCLAIMS ALL WARRANTIES
 * WITH  REGARD   TO  THIS  SOFTWARE  INCLUDING  ALL   IMPLIED  WARRANTIES  OF
 * MERCHANTABILITY AND  FITNESS.  IN NO EVENT  SHALL THE AUTHOR  BE LIABLE FOR
 * ANY  SPECIAL, DIRECT,  INDIRECT, OR  CONSEQUENTIAL DAMAGES  OR  ANY DAMAGES
 * WHATSOEVER  RESULTING FROM  LOSS OF  USE, DATA  OR PROFITS,  WHETHER  IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR  OTHER TORTIOUS ACTION, ARISING OUT OF OR
 * IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *                                           Anthony Mallet on Mon Aug 20 2018
 */
#include "acmccdaqft.h"

#include <Eigen/Core>
#include <Eigen/Dense>

#include "mccdaqft_c_types.h"
#include "codels.h"

void
daqft_get_calibration(const or_t3d_pos *pos, const or_t3d_att *att,
                      const double calibration[36], const double scale[6],
                      const or_t3d_pos *p, const or_t3d_att *q,
                      double G[36])
{
  using namespace Eigen;

  const Matrix<double, 3, 1> poffset(pos->x, pos->y, pos->z);
  const Quaternion<double> qoffset(att->qw, att->qx, att->qy, att->qz);

  const Matrix<double, 3, 1> p_(p->x, p->y, p->z);
  const Quaternion<double> q_(q->qw, q->qx, q->qy, q->qz);

  Map< const Matrix<double, 6, 6, RowMajor> > calibration_(calibration);
  Map< const Array<double, 6, 1> > scale_(scale);
  Map< Matrix<double, 6, 6, RowMajor> > G_(G);

  Matrix<double, 6, 6, RowMajor> offset;

  G_.block<3,3>(0, 0) = qoffset.matrix();
  G_.block<3,3>(0, 3) = Matrix<double, 3, 3>::Zero();
  G_.block<3,3>(3, 0) = qoffset.matrix().rowwise().cross(-poffset);
  G_.block<3,3>(3, 3) = qoffset.matrix();

  offset.block<3,3>(0, 0) = q_.matrix();
  offset.block<3,3>(0, 3) = Matrix<double, 3, 3>::Zero();
  offset.block<3,3>(3, 0) = q_.matrix().rowwise().cross(-p_);
  offset.block<3,3>(3, 3) = q_.matrix();

  G_ = offset * G_ * (calibration_.array().colwise() / scale_).matrix();
}

void
daqft_get_data(const double data[6], const double G[36], const double b[6],
               double w[6])
{
  using namespace Eigen;

  Map< const Matrix<double, 6, 1> > data_(data);
  Map< const Matrix<double, 6, 6, RowMajor> > G_(G);
  Map< const Matrix<double, 6, 1> > b_(b);
  Map< Matrix<double, 6, 1> > w_(w);

  w_ = G_ * (data_ - b_);
}
